unit Datacole;

{*
 *                     tDataCollector VCL Component
 *                            Version 3.0a
 *
 *                         16bit and 32bit code
 *
 *            Copyright 1996 Absolute Computer Consulting
 *                        All Rights Reserved
 *                          03 October 1996
 *
 *             Please see datacole.hlp for more information.
 *
 *   This VCL component and the associated files can be freely used and
 *   distributed in commercial and private environments, provided this
 *   notice is not modified in any way without my expressed written
 *   consent.  The tTableDBF component is released as is.  No warranty
 *   is implied and we are not responsible for any problems that might
 *   arise from the use of this unit.  You use this unit entirely at
 *   your own risk.
 *}

(**) interface (**)

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, Grids, DBGrids, ExtCtrls, DB, DBTables,
  DBCtrls, Menus, Tabs;

const
  DefaultCaption = 'Data searching / reporting tool by ACC';
  tDCBadSource   = 'tDataCollector:  You did not specify a source table.';

type

  { Define a forward declaration for this class. }
  tDataCollector = class;

  {
    This is the declaration for the user-interface part of the component.
    This form will be called by the tDataCollector component.
  }
  TfrmSetupQuery = class(TForm)
    DataSource1: TDataSource;
    Panel3: TPanel;
    Panel4: TPanel;
    DBGrid1: TDBGrid;
    PreviewData: TBitBtn;
    Panel1: TPanel;
    DBNavigator1: TDBNavigator;
    SQLStatus: TPanel;
    PrintPreview: TBitBtn;
    PrintReport: TBitBtn;
    RefGrid: TStringGrid;
    MainMenu1: TMainMenu;
    Close1: TMenuItem;
    AddCondition1: TMenuItem;
    ManuallysetSQLcriteria1: TMenuItem;
    DeleteCondition1: TMenuItem;
    Panel5: TPanel;
    Panel6: TPanel;
    Panel2: TPanel;
    WhereClause: TMemo;
    TabSet: TTabSet;
    Notebook: TNotebook;
    DataGrid: TStringGrid;
    DataGrid2: TStringGrid;
    DataGrid3: TStringGrid;
    DataGrid4: TStringGrid;
    DataGrid5: TStringGrid;
    DataGrid6: TStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure PreviewDataClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure PrintPreviewClick(Sender: TObject);
    procedure PrintReportClick(Sender: TObject);
    procedure DataGridKeyPress(Sender: TObject; var Key: Char);
    procedure DataGridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Close1Click(Sender: TObject);
    procedure AddCondition1Click(Sender: TObject);
    procedure TabSetClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ManuallysetSQLcriteria1Click(Sender: TObject);
    procedure DeleteCondition1Click(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateSQLDisplayLabels;
  public
    { Public declarations }
    GridList   : tList;
    Conditions : integer;
    TableLine  : string;
    OrderLine  : string;
    DataQuery  : TQuery;
    Collector  : tDataCollector;
    procedure BuildConditions;
    procedure BuildQuery;
    procedure ResetGrid;
    procedure SetGridFromTable(DataSet : tDataSet);
  end;

  {
    Define event proto-type for our events.
  }
  TPrintReportEvent = procedure (DataSet : tDataSet; DataSource : tDataSource) of object;
  TAfterSQLEvent    = procedure (Sender : tObject; DataGrid : tStringGrid; DataSet : tQuery) of object;
  TScreenGridEvent  = procedure (DataGrid : tStringGrid) of object;

  {
    This is the definition of the tDataCollector component.  It is this
    component that is installed in your component palette.
  }
  TDataCollector = class(TComponent)
  private
    { Private declarations }
    FOnReportPrint   : tPrintReportEvent;
    FOnReportPreview : tPrintReportEvent;
    FBeforeSQLEvent  : TScreenGridEvent;
    FAfterSQLEvent   : tAfterSQLEvent;
    FTableSetup      : tScreenGridEvent;
    FDebugMode       : boolean;
    FAllowOR         : boolean;
    FShowPrint       : boolean;
    FSplitDates      : boolean;
    FWinCaption      : string;
    FFields          : string;
    FSort            : string;
    FPrintCaption    : string;
    FGrid            : tStringGrid;
    FQuery           : tQuery;
    FTable           : tTable;         { Source table                         }
    fHelpId          : tHelpContext;   { Help context.                        }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner : tComponent); override;
    function    Execute : boolean;
    procedure   CheckBeforeSQL;
    procedure   CheckAfterSQL;
    procedure   RebuildSQL;
    function    GetDataGrids(var Count : integer) : tList;
    property    ResultQuery     : tQuery         read FQuery;
    property    DataGrid        : tStringGrid    read FGrid   write FGrid;
  published
    { Published declarations }
    property DebugMode       : boolean           read    FDebugMode
                                                 write   FDebugMode;
    property AllowOR         : boolean           read    FAllowOR
                                                 write   FAllowOR;                    
    property ShowPrintButtons: boolean           read    FShowPrint
                                                 write   FShowPrint;
    property WindowCaption   : string            read    FWinCaption
                                                 write   FWinCaption;
    property SQLFields       : string            read    FFields
                                                 write   FFields;
    property SQLSortOrder    : string            read    FSort
                                                 write   FSort;
    property SplitDates      : boolean           read    FSplitDates
                                                 write   FSplitDates
                                                 default FALSE;
    property PrintCaption    : string            read    FPrintCaption
                                                 write   FPrintCaption;
    property SourceTable     : tTable            read    FTable
                                                 write   FTable;
    property HelpContext     : tHelpContext      read    fHelpId
                                                 write   fHelpId;
    property BeforeSQL       : TScreenGridEvent  read    FBeforeSQLEvent
                                                 write   FBeforeSQLEvent;
    property AfterSQL        : TAfterSQLEvent    read    FAfterSQLEvent
                                                 write   FAfterSQLEvent;
    property OnReportPrint   : TPrintReportEvent read    FOnReportPrint
                                                 write   FOnReportPrint;
    property OnReportPreview : TPrintReportEvent read    FOnReportPreview
                                                 write   FOnReportPreview;
    property TableSetup      : tScreenGridEvent  read    FTableSetup
                                                 write   FTableSetup;
  end;

procedure Register;

var
  frmSetupQuery: TfrmSetupQuery;

implementation

{$R *.DFM}

{ ------------------------------------------------------------------------ }
{ -------  Utility function for formatting strings  ---------------------- }
{ ------------------------------------------------------------------------ }

{$ifndef win32}

  function RTrimASM(InString : String) : String; assembler;
  asm
     push ds
     cld
     lds si, [InString]                     { Load INSTRING to DS:SI       }
     les di, @Result                        { Load Result to ES:DI         }
     lodsb                                  { get len of string, 1st byte  }
     push si                                { Store where first byte is    }
     xor ah,ah                              { word-to-byte                 }
     mov cx, ax                             { Place len of string into cx  }
     jcxz @@Error                           { empty string, jump to bottom }

     std                                    { set direction flag           }
                                            { ---------------------------- }
                                            { Now we have to start at      }
                                            { far right and end up left    }
                                            { ---------------------------- }
     add si, cx                             { move ptr to end of string    }
     dec si                                 { When adding, 1 too many      }
    @@Continue:
     lodsb                                  { load next char from scr str  }
     cmp al, $20                            { compare char to <space>      }
     jne @@Done                             { It's not space, STOP         }
     loop @@Continue

    @@Done:
     cld                                    { Clear direction flag         }
     mov ax, cx                             { put length of result in ax   }
     xor ah, ah                             { word-to-byte                 }
     pop si                                 { reset source to start of str }
     stosb                                  { write length to string       }
     rep movsb                              { write rest of string         }
     jmp @@Finished

    @@Error:
     pop si                                 { release the stack used..     }
     mov al, 0                              { return a null for length     }
     stosb                                  { store that byte              }

    @@Finished:
     pop ds
  end; { Function RTrimASM }

{ ------------------------------------------------------------------------ }

  function LTrimASM(InString : String) : String; assembler;
                              { Take all the right spaces off a string.... }
  asm
     push ds
     cld
     lds si, [InString]                     { Load INSTRING to DS:SI       }
     lodsb                                  { get len of string, 1st byte  }
     les di, @Result                        { Load Result to ES:DI         }
     xor ah,ah                              { word-to-byte                 }
     mov cx, ax                             { Place len of string into cx  }
     jcxz @@Error                           { empty string, jump to bottom }

    @@Continue:
     lodsb                                  { load byte from InString      }
     cmp al, $20                            { Compare, is it a space??     }
     jne @@Done                             { Not Space, time STOP.        }
     loop @@Continue

    @@Done:
     mov ax, cx                             { Get ready to store length    }
     xor ah, ah                             { of new string in RESULT      }
     stosb                                  { Store length of string       }
     dec si                                 { We were one to far ...       }
     rep movsb                              { We weeded out the 1st spaces }
                                            { Now store rest of string     }
                                            { in Result                    }
     jmp @@Finished

    @@Error:
     mov al, 0                              { return a null for length     }
     stosb                                  { store that byte              }

    @@Finished:
     pop ds
  end; { Procedure LTrim }

{ ------------------------------------------------------------------------ }

  function AllTrim(InString : STRING) : STRING;
  begin
    InString := rTrimASM(InString);
    InString := lTrimASM(InString);
    AllTrim  := InString;
  end;

{$else}

  function AllTrim(InString: String) : String;
  begin
    Result := Trim(InString);
  end;

{$endif}

{ --------------------------------------------------------------------------- }
{ -------  tDataCollector Code starts here  --------------------------------- }
{ --------------------------------------------------------------------------- }

constructor TDataCollector.Create(AOwner : tComponent);
begin

  inherited Create(AOwner);
  {
    Define the inital state for a new tDataCollector instance.
  }
  fAllowOR      := FALSE;
  fDebugMode    := FALSE;
  fShowPrint    := FALSE;
  FPrintCaption := 'Print &Report';
  fWinCaption   := DefaultCaption;
  fSort         := '';
  fFields       := '*';
  fSplitDates   := FALSE;
  fTable        := NIL;
  fHelpID       := 0;
  {
    Since the tQuery is attached to this instance of a tDataCollector class,
    we do not need any code to dispose of it.  When the tDataCollector
    instance is destroyed, the tQuery that is attached to it should also
    be destroyed.
  }
  FQuery        := tQuery.Create(Self);

end;

{ --------------------------------------------------------------------------- }

{
  This is the main function that start the ball rolling for the component.
}

function TDataCollector.Execute : boolean;
begin
  Result        := TRUE;
  fQuery.Active := FALSE;

  { Create our new form. }
  frmSetupQuery := TfrmSetupQuery.Create(nil);
  try
    with frmSetupQuery do
    begin

      { Setup the condtions. }
      if not fAllowOR then begin
        AddCondition1.Visible    := FALSE;
        DeleteCondition1.Visible := FALSE;
      end;

      {
        Setup the information in the form.
      }
      fGrid               := DataGrid;
      Caption             := fWinCaption;
      HelpContext         := fHelpID;
      PrintReport.Caption := FPrintCaption;
      Collector           := SELF;

      if not Assigned(fTable) then
      begin
        { The programmer didn't specify a SourceTable.  Raise an exception. }
        raise Exception.Create(tDCBadSource);
      end;

      if fTable.DatabaseName <> '' then
        FQuery.DatabaseName := fTable.DatabaseName;

      {
        Make sure the table name in enclosed in " ".  This is stupid
        requirement of the BDE.  If the table name is not in quotes and the
        user specifies a full path of a file name, the SQL will fail.
      }
      TableLine := format('select %s from "%s"', [fFields, ChangeFileExt(fTable.TableName,'')]);
      OrderLine := 'order by ' + fSort;

      { Setup the string grids with the information we need. }
      SetGridFromTable(fTable);

      {
        Call the event TableSetup if and only if the user created one.
      }
      if assigned(fTableSetup) then fTableSetup(DataGrid);

      DataQuery := fQuery;

      PrintReport.Visible  := FALSE;
      PrintPreview.Visible := FALSE;

      {
        Decide if the PREVIEW and PRINT buttons should be visible to the
        user.
      }
      if fShowPrint = TRUE then
      begin
        if Assigned(FOnReportPrint)   then PrintReport.Visible  := TRUE;
        if Assigned(FOnReportPreview) then PrintPreview.Visible := TRUE;
      end;

      { Show the form and let the user see / use this component. }
      ShowModal;

    end;
  finally
    { We are done with the form, destroy it. }
    frmSetupQuery.Free;
  end;

end;

{ --------------------------------------------------------------------------- }

procedure TDataCollector.CheckBeforeSQL;
begin
  if assigned(fBeforeSQLEvent) then fBeforeSQLEvent(frmSetupQuery.DataGrid);
end;

{ --------------------------------------------------------------------------- }

procedure TDataCollector.CheckAfterSQL;
begin
  if assigned(fAfterSQLEvent) then fAfterSQLEvent(Self, frmSetupQuery.DataGrid, ResultQuery);
end;

{ --------------------------------------------------------------------------- }

procedure TDataCollector.RebuildSQL;
begin
  {
    This is a public routine that can be called by the programmer to force the
    rebuilding of the SQL that is used to extract data from the source table.
  }
  frmSetupQuery.BuildQuery;
end;

{ --------------------------------------------------------------------------- }

function TDataCollector.GetDataGrids(var Count : integer) : tList;
begin
  Count  := frmSetupQuery.Conditions;
  Result := frmSetupQuery.GridList;
end;

{ ------------------------------------------------------------------------ }
{ -------  TfrmSetupQuery code starts here  ------------------------------ }
{ ------------------------------------------------------------------------ }

procedure TfrmSetupQuery.FormCreate(Sender: TObject);
var
  i : integer;
begin

  { Define the initial number of conditions. }
  Conditions := 1;

  {
    Build a tList to hold pointers to the string grids that are used on
    this form.  The main reason for collecting them in a tList is so that
    we can use a for loop to access all the grids.
  }
  GridList := tList.Create;
  GridList.Add(DataGrid);
  GridList.Add(DataGrid2);
  GridList.Add(DataGrid3);
  GridList.Add(DataGrid4);
  GridList.Add(DataGrid5);
  GridList.Add(DataGrid6);

  {
    Set the column heading in the string grids.
  }
  for i := 0 to GridList.Count - 1 do begin
    With tStringGrid(GridList.Items[i]) do
    begin
      ColWidths[0] := DefaultColWidth + 20;
      ColWidths[1] := DefaultColWidth - 20;

      Cells[0,0] := 'Please enter criteria';
      Cells[1,0] := '        Example';
      Cells[2,0] := '          Equal to';
      Cells[3,0] := '      Greater than (>=)';
      Cells[4,0] := '        Less than (<=)';
    end;
  end;

end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.ResetGrid;
var
  i : integer;
  x : integer;
  y : integer;
begin

 {
   Use a quick and dirty routine to clear the two string grids.
 }

 for i := 0 to GridList.Count - 1 do begin
   With tStringGrid(GridList.Items[i]) do begin

    for x := 1 to (RowCount-1) do
     for y := 0 to (ColCount-1) do
       Cells[y,x] := '';

    Row := 1;
    Col := 2;
   end; { with }
 end;

 with RefGrid do
  for x := 1 to (RowCount-1) do
   for y := 0 to (ColCount-1) do
     Cells[y,x] := '';

end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.SetGridFromTable(DataSet : tDataSet);
var
  i, x, y : integer;
begin

  {
    This routine will fill the strings grids with information form
    the source table.

    If the field in the source is not visible, it is not added to the
    string grids.

    Current we handle:  tDateField, tFloatField, and tStringField.  With
    the tStringField, if the size of the field is larger than 10, a LIKE
    search is done.
  }

  with DataGrid do
  begin

    y := 0;
    for x := 0 to (DataSet.FieldCount-1) do
    begin
      if DataSet.Fields[x].Visible = TRUE then
      begin
        Cells[0,y+1] := DataSet.Fields[x].DisplayLabel;

        if DataSet.Fields[x] is tDateField then
          Cells[1,y+1] := DateToStr(DataSet.Fields[x].AsDateTime)
        else
          Cells[1,y+1] := DataSet.Fields[x].AsString;

        RefGrid.Cells[0,y+1] := DataSet.Fields[x].FieldName;
        if DataSet.Fields[x] is tFloatField then RefGrid.Cells[1,y+1] := 'N';
        if DataSet.Fields[x] is tDateField  then
        begin
          if Collector.SplitDates then
          begin
            RefGrid.Cells[0,y+1] := DataSet.Fields[x].FieldName;
            RefGrid.Cells[0,y+2] := DataSet.Fields[x].FieldName;
            RefGrid.Cells[0,y+3] := DataSet.Fields[x].FieldName;
            RefGrid.Cells[0,y+4] := DataSet.Fields[x].FieldName;

            RefGrid.Cells[1,y+1] := 'D';   { Whole year }
            RefGrid.Cells[1,y+2] := 'DY';  { year extracted from date  }
            RefGrid.Cells[1,y+3] := 'DM';  { month extracted from date }
            RefGrid.Cells[1,y+4] := 'DD';  { day extracted from date   }

            Cells[0,y+2] := format('%s (YEAR)', [DataSet.Fields[x].DisplayLabel]);
            Cells[1,y+2] := FormatDateTime('YYYY', DataSet.Fields[x].AsDateTime);

            Cells[0,y+3] := format('%s (MONTH)', [DataSet.Fields[x].DisplayLabel]);
            Cells[1,y+3] := FormatDateTime('MM', DataSet.Fields[x].AsDateTime);

            Cells[0,y+4] := format('%s (DAY)', [DataSet.Fields[x].DisplayLabel]);
            Cells[1,y+4] := FormatDateTime('DD', DataSet.Fields[x].AsDateTime);

            inc(y, 3);
          end
          else
            RefGrid.Cells[1,y+1] := 'D';
        end;

        if DataSet.Fields[x] is tStringField then
         if DataSet.Fields[x].Size >= 10 then RefGrid.Cells[1,y+1] := 'L';

        RefGrid.Cells[2,y+1] := ChangeFileExt((DataSet as tTable).TableName, '');
        inc(y);
      end;

    end;

    DataGrid.RowCount := y + 1;
  end; { with }

  { We need to copy this information to the other grids. }
  for i := 1 to GridList.Count - 1 do begin
    with tStringGrid(GridList.Items[i]) do begin

      for x := 1 to DataGrid.RowCount - 1 do
        for y := 0 to DataGrid.ColCount - 1 do
          Cells[y,x] := DataGrid.Cells[y,x];

      RowCount := DataGrid.RowCount;

    end;
  end; { for }

end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.BuildConditions;
var
  i       : integer;
  x       : integer;
  z       : integer;
  op      : string[2];
  First   : boolean;
  TmpGrid : tStringGrid;

  { ------------------------------------------------------------------------- }
  procedure AddSQL;
  var
    DataValue  : string;
    DateValue  : tDateTime;
    StartValue : string[10];
    TmpStr     : string[4];
  begin

   DataValue := AnsiUpperCase(Alltrim(TmpGrid.Cells[z,x]));
   if Length(DataValue) = 0 then exit;

   with TmpGrid do
   if z <> -1 then
   begin
    if First = TRUE then
    begin

      if i = 0 then
        StartValue := 'where ('
      else
        StartValue := 'or (';

      First := FALSE;
    end
    else
    begin
      StartValue := 'and';
    end; { first }

    TmpStr := RefGrid.Cells[1,x];
    if TmpStr = 'N' then
    begin
     WhereClause.Lines.Add( format('%s %s %s %s',
                        [StartValue, RefGrid.Cells[0,x], op, DataValue]) );
    end
    else if TmpStr = 'L' then
    begin
     WhereClause.Lines.Add( format('%s UPPER(%s) like "%%%s%%"',
                        [StartValue, RefGrid.Cells[0,x], DataValue]));
    end
    else if TmpStr = 'D' then
    begin
      DateValue := StrToDate(DataValue);
      DataValue := FormatDateTime('mm"/"dd"/"yyyy', DateValue);
      WhereClause.Lines.Add( format('%s %s %s "%s"',
                         [StartValue, RefGrid.Cells[0,x], op, DataValue]) );
    end
    else if TmpStr = 'DY' then
    begin
      WhereClause.Lines.Add( format('%s extract(year from %s) %s %s',
                         [StartValue, RefGrid.Cells[0,x], op, DataValue]) );
    end
    else if TmpStr = 'DM' then
    begin
      WhereClause.Lines.Add( format('%s extract(month from %s) %s %s',
                         [StartValue, RefGrid.Cells[0,x], op, DataValue]) );
    end
    else if TmpStr = 'DD' then
    begin
      WhereClause.Lines.Add( format('%s extract(day from %s) %s %s',
                         [StartValue, RefGrid.Cells[0,x], op, DataValue]) );
    end
    else
    begin
      WhereClause.Lines.Add( format('%s upper(%s) %s "%s"',
                         [StartValue, RefGrid.Cells[0,x], op, DataValue]) );
    end; { case }

   end; { if z <> -1 }
  end;

  { ------------------------------------------------------------------------- }

begin

  WhereClause.Clear;

  for i := 0 to TabSet.Tabs.Count - 1 do begin

    TmpGrid := tStringGrid(GridList.Items[i]);
    First   := TRUE;

    for x := 1 to (TmpGrid.RowCount-1) do
    begin
      z := -1;
      with TmpGrid do
      begin
        if RefGrid.Cells[0,x] <> '' then   { we have a valid field name           }
        begin
          if Cells[2,x] <> '' then         { we have an EQUALs set.               }
          begin
            op := '=';
            z  := 2;
            AddSQL;
          end
          else
          begin
            if (Cells[3,x] <> '') then     { we have a less than set.             }
            begin
              op := '>=';
              z  := 3;
              AddSQL;
            end;
            if (Cells[4,x] <> '') then     { we have a greater than set.          }
            begin
              op := '<=';
              z  := 4;
              AddSQL;
            end;
          end; { if Cells[2,x] }
        end;
      end;

    end; { for }

    if WhereClause.Lines.Count > 0 then
      WhereClause.Lines.Add(')');

  end; { for }

end;

{ --------------------------------------------------------------------------- }


{
  This routine will take the information the user entered into the grid and
  create an SQL statement to extract the data.
}

procedure TfrmSetupQuery.BuildQuery;
var
  i : integer;
begin

  With DataQuery do
  begin
    SQL.Clear;
    SQL.Add(TableLine);
  end;

  if not WhereClause.Visible then BuildConditions;

  { Add the where clause }
  if WhereClause.Lines.Count > 0 then
    if WhereClause.Lines[0] <> '' then
      for i := 0 to WhereClause.Lines.Count - 1 do
        DataQuery.SQL.Add(WhereClause.Lines[i]);


  if AllTrim(OrderLine) <> 'order by' then DataQuery.SQL.Add( OrderLine );
  if (Collector.DebugMode = TRUE) then DataQuery.SQL.SaveToFile('dcdebug.txt');

end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.PreviewDataClick(Sender: TObject);
begin

  BuildQuery;                          { Build the SQL                        }

  Screen.Cursor := crSQLWait;
  try
    Collector.CheckBeforeSQL;          { is there a PRE SQL event?            }
    DataQuery.Open;                    { execute the SQL and open the result.  }
    UpdateSQLDisplayLabels;            { Let's update the display labels.     }
    Collector.CheckAfterSQL;           { is there a POST SQL event?           }
    DataSource1.DataSet := DataQuery;
    { Set the status bar. }
    if DataQuery.RecordCount = 1 then
      SQLStatus.Caption := 'There is 1 match to your query'
    else
      SQLStatus.Caption := format('There are %d matches to your query', [DataQuery.RecordCount]);

    { Enable the buttons, we have data extracted from the source table. }
    PrintPreview.Enabled := True;
    PrintReport.Enabled  := True;
  finally
    Screen.Cursor := crDefault;
  end;

end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.FormActivate(Sender: TObject);
begin
  DataQuery.Active     := FALSE;
  SQLStatus.Caption    := 'No active data set';
  PrintPreview.Enabled := FALSE;
  PrintReport.Enabled  := FALSE;
  DataGrid.SetFocus;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.PrintPreviewClick(Sender: TObject);
begin
  { code to handle PrintPreviews }
  with Collector do
  begin
    DBGrid1.DataSource := NIL;
    OnReportPreview(DataQuery, DataSource1);
    DBGrid1.DataSource := DataSource1;
  end;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.PrintReportClick(Sender: TObject);
begin
  { code to handle printing reports }
  with Collector do
  begin
    DBGrid1.DataSource := NIL;
    OnReportPrint(DataQuery, DataSource1);
    DBGrid1.DataSource := DataSource1;
  end;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.DataGridKeyPress(Sender: TObject; var Key: Char);
begin
  { We want to disable the Print Preview and Print Report buttons. }
  PrintPreview.Enabled := FALSE;
  PrintReport.Enabled  := FALSE;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.DataGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  TheGrid : tStringGrid;
begin
  TheGrid := Sender as tStringGrid;
  { Set the grid up so that the enter key acts like a down cursor. }
  if Key = VK_Return then
  begin
    TheGrid.Row  := (TheGrid.Row + 1) mod (TheGrid.RowCount - 1);
  end;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.UpdateSQLDisplayLabels;
var
  x, y  : integer;
  Found : boolean;
begin

  {
    If the FieldCount of the source is the same as the destination, we
    are going to copy the DisplayLabel, DisplayWidth, and Alignment
    properties from the source table to the SQL output.
  }

  for x := 0 to (DataQuery.FieldCount - 1) do
  begin

    y := 0; Found := FALSE;
    while (y < Collector.SourceTable.FieldDefs.Count - 1) AND (FOUND = FALSE) do
    begin
      try
        Found := Collector.SourceTable.Fields[y].FieldName =
                 DataQuery.Fields[x].FieldName;
      except
        {
          For some reason, if the programmer has only defined a few fields.
          The field count is still reported as the actual number of fields
          in the table.  BUT, when you try to access the fields property
          you get a List Index out of Bounds error.  We have added this
          exception handler to fix this problem.  Hopefully, we will be
          able to find a better fix.

          If you have a better fix email it to me!! (klaamasm@tuns.ca)
        }
        Found := FALSE;
        y     := Collector.SourceTable.FieldDefs.Count;
      end;
      if NOT Found then inc(y);
    end;

    if Found then
    begin
      DataQuery.Fields[x].Alignment    := Collector.SourceTable.Fields[y].Alignment;
      DataQuery.Fields[x].DisplayWidth := Collector.SourceTable.Fields[y].DisplayWidth;
      DataQuery.Fields[x].DisplayLabel := Collector.SourceTable.Fields[y].DisplayLabel;
    end;
  end;

end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.Close1Click(Sender: TObject);
begin
  { The user click on CLOSE, let's just do that for him ... }
  Close;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.AddCondition1Click(Sender: TObject);
var
  TmpGrid : tStringGrid;
  x, y    : integer;
begin

  if TabSet.Tabs.Count < 6 then begin
    TabSet.Tabs.Add(format('Condition &%d', [TabSet.Tabs.Count+1]));

    NoteBook.PageIndex := TabSet.Tabs.Count-1;
    TabSet.TabIndex    := NoteBook.PageIndex;
    inc(Conditions);
    { Copy data from primary data grid. }
    TmpGrid := (GridList.Items[Conditions-1]);

    for x := 1 to TmpGrid.RowCount - 1 do
      for y := 2 to TmpGrid.ColCount - 1 do
        TmpGrid.Cells[y, x] := DataGrid.Cells[y, x];

  end;

end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.TabSetClick(Sender: TObject);
begin
  NoteBook.PageIndex := TabSet.TabIndex;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.FormDestroy(Sender: TObject);
begin
  GridList.Free;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.ManuallysetSQLcriteria1Click(Sender: TObject);
begin
  WhereClause.Visible := not WhereClause.Visible;
  NoteBook.Visible    := not NoteBook.Visible;
end;

{ --------------------------------------------------------------------------- }

procedure TfrmSetupQuery.DeleteCondition1Click(Sender: TObject);
begin
  if TabSet.Tabs.Count > 1 then begin
    TabSet.Tabs.Delete(TabSet.Tabs.Count-1);
    dec(Conditions);
  end;
end;

{ --------------------------------------------------------------------------- }
{ -------  Code to register the component  ---------------------------------- }
{ --------------------------------------------------------------------------- }

procedure Register;
begin
  {
    This procedure will register the tDataCollector component and create
    a "icon" for it in the MKTools palette.
  }
  RegisterComponents('MKTools', [TDataCollector]);
end;

{ --------------------------------------------------------------------------- }

end.
